/*
	$Id: symbolsystem.c,v 1.2 2004/05/09 03:17:03 syl Exp $

	Copyright (C) 2004 Syllabub
	Maroontress Fast Software.
*/

#include <stdlib.h>
#include <string.h>

#include "WCString.h"
#include "symbolsystem.h"

typedef struct ConversionRule {
    struct ConversionRule *next;
    wchar_t *src_wcs;
    int src_len;
    wchar_t *dst_wcs;
    int dst_len;
} ConversionRule;

static wchar_t *
wcs_create(const char *mbs)
{
    int bytes, len;
    wchar_t *wcs;

    if (mbs == NULL)
	return (NULL);
    bytes = strlen(mbs) + 1;
    if ((wcs = (wchar_t *)alloca(sizeof(wchar_t) * bytes)) == NULL
        || (len = mbstowcs(wcs, mbs, bytes)) < 0) {
        return (NULL);
    }
    return (wstrdup(wcs));
}

static void
FreeConversionRule(ConversionRule *top)
{
    ConversionRule *next;

    for (; top != NULL; top = next) {
        next = top->next;
        free(top->src_wcs);
        free(top->dst_wcs);
        free(top);
    }
}

static ConversionRule *
AddToConversionRule(ConversionRule *top, const char *src, const char *dst)
{
    ConversionRule *map;
    wchar_t *wcs;

    if ((map = (ConversionRule *)malloc(sizeof(ConversionRule))) == NULL)
        return (NULL);
    map->next = NULL;
    map->src_wcs = NULL;
    map->dst_wcs = NULL;

    if ((wcs = wcs_create(src)) == NULL) {
        FreeConversionRule(map);
        return (NULL);
    }
    map->src_len = wstrlen(wcs);
    map->src_wcs = wcs;

    if ((wcs = wcs_create(dst)) == NULL) {
        FreeConversionRule(map);
        return (NULL);
    }
    map->dst_len = wstrlen(wcs);
    map->dst_wcs = wcs;

    map->next = top;
    return (map);
}

static int
GetCharPosition(wchar_t *wcs, wchar_t c)
{
    int n;

    for (n = 0; wcs[n] != 0 && wcs[n] != c; ++n)
        ;
    return (n);
}

static wchar_t
Replacer(ConversionRule *r, wchar_t c)
{
    int pos;

    for (; r != NULL && (pos = GetCharPosition(r->src_wcs, c)) >= r->src_len;
         r = r->next) {
        continue;
    }
    return ((r == NULL || pos >= r->dst_len) ? c : r->dst_wcs[pos]);
}

static ConversionRule *symbolRule = NULL;

wchar_t *
ReplaceSymbolSystem(wchar_t *wcs)
{
    int k, len = wstrlen(wcs);

    for (k = 0; k < len; ++k) {
        wcs[k] = Replacer(symbolRule, wcs[k]);
    }
    return wcs;
}

void
InitializeSylbolSystem(void)
{
    const char *hira = "" ""
	"" "¤äĤŤƤǤȤ" "ʤˤ̤ͤ"
	"ϤФѤҤӤԤդ֤פؤ٤ڤۤܤ" "ޤߤ" ""
	"" "";
    const char *kata = "" ""
	"" "¥åĥťƥǥȥ" "ʥ˥̥ͥ"
	"ϥХѥҥӥԥե֥ץإ٥ڥۥܥ" "ޥߥ" ""
	"" "";

    symbolRule = AddToConversionRule(symbolRule, kata, hira);
}

void
FinalizeSylbolSystem(void)
{
    FreeConversionRule(symbolRule);
}
